@@ -24,7 +24,7 @@ was written by Jeff Waugh and is available at www.perkypants.org
 #   Ian Weller <ianweller@gmail.com>
 #
 # Released under the terms of the GNU GPL
-# http://www.gnu.org/copyleft/gpl.html
+# https://www.gnu.org/copyleft/gpl.html
 
 # Differences from the Perl version:
 #   There are no hardcoded nick colour preferences for jdub, cantanker and
@@ -49,19 +49,22 @@ from __future__ import print_function, unicode_literals
 
 import gzip
 import io
+import itertools
 import optparse
 import os
 import re
 import shlex
 import shutil
 import sys
+import datetime
 
 try:
     from urllib import quote
 except ImportError:
     from urllib.parse import quote
 
-from ._version import __version__ as VERSION, __date__ as RELEASE
+from ._version import (
+    __version__ as VERSION, __date__ as RELEASE, __homepage__ as HOMEPAGE)
 
 try:
     unicode
@@ -113,24 +116,26 @@ class LogParser(object):
     OTHER = Enum('OTHER')
 
     TIME_REGEXP = re.compile(
-            r'^\[?(' # Optional [
-            r'(?:\d{4}-\d{2}-\d{2}T|\d{2}-\w{3}-\d{4} |\w{3} \d{2} |\d{2} \w{3} )?' # Optional date
-            r'\d\d:\d\d(:\d\d)?' # Mandatory HH:MM, optional :SS
-            r')\]? +') # Optional ], mandatory space
-    NICK_REGEXP = re.compile(r'^<(.*?)(!.*)?>\s')
+        r'^\[?(' # Optional [
+        r'(?:\d{4}-\d{2}-\d{2}T|\d{2}-\w{3}-\d{4} |\w{3} \d{2} |\d{2} \w{3} )?' # Optional date
+        r'\d\d:\d\d(:\d\d)?' # Mandatory HH:MM, optional :SS
+        r')\]? +') # Optional ], mandatory space
+    TIMESTAMP_REGEXP = re.compile(r'^(\d+) +')
+    NICK_REGEXP = re.compile(r'^<(.*?)(!.*?)?>\s')
     DIRCPROXY_NICK_REGEXP = re.compile(r'^<(.*?)(!.*)?>\s[\+-]?')
     JOIN_REGEXP = re.compile(r'^(?:\*\*\*|-->)\s.*joined')
     PART_REGEXP = re.compile(r'^(?:\*\*\*|<--)\s.*(quit|left)')
     SERVMSG_REGEXP = re.compile(r'^(?:\*\*\*|---)\s')
     NICK_CHANGE_REGEXP = re.compile(
-            r'^(?:\*\*\*|---)\s+(.*?) (?:are|is) now known as (.*)')
+        r'^(?:\*\*\*|---)\s+(.*?) (?:are|is) now known as (.*)')
 
     def __init__(self, infile, dircproxy=False):
         self.infile = infile
         if dircproxy:
             self.NICK_REGEXP = self.DIRCPROXY_NICK_REGEXP
 
-    def decode(self, s):
+    @staticmethod
+    def decode(s):
         """Convert 8-bit string to Unicode.
 
         Supports xchat's hybrid Latin/Unicode encoding, as documented here:
@@ -157,6 +162,13 @@ class LogParser(object):
             else:
                 time = None
 
+            if time is None:
+                m = self.TIMESTAMP_REGEXP.match(line)
+                if m:
+                    time = datetime.datetime.utcfromtimestamp(
+                        int(m.group(1))).isoformat()
+                    line = line[len(m.group(0)):]
+
             m = self.NICK_REGEXP.match(line)
             if m:
                 nick = m.group(1)
@@ -173,7 +185,6 @@ class LogParser(object):
                 if m:
                     oldnick = m.group(1)
                     newnick = m.group(2)
-                    line = line
                     yield time, self.NICKCHANGE, (line, oldnick, newnick)
                 elif self.SERVMSG_REGEXP.match(line):
                     yield time, self.SERVER, line
@@ -218,7 +229,7 @@ def shorttime(time):
 class ColourChooser:
     """Choose distinguishable colours."""
 
-    def __init__(self, rgbmin=240, rgbmax=125, rgb=None, a=0.95, b=0.5):
+    def __init__(self, rgbmin=240, rgbmax=125, a=0.95, b=0.5):
         """Define a range of colours available for choosing.
 
         `rgbmin` and `rgbmax` define the outmost range of colour depth (note
@@ -237,16 +248,16 @@ class ColourChooser:
         assert 0 <= rgbmax < 256
         self.rgbmin = rgbmin
         self.rgbmax = rgbmax
-        if not rgb:
-            assert 0 <= a <= 1.0
-            assert 0 <= b <= 1.0
-            rgb = [(a,b,b), (b,a,b), (b,b,a), (a,a,b), (a,b,a), (b,a,a)]
-        else:
-            for r, g, b in rgb:
-                assert 0 <= r <= 1.0
-                assert 0 <= g <= 1.0
-                assert 0 <= b <= 1.0
-        self.rgb = rgb
+        assert 0 <= a <= 1.0
+        assert 0 <= b <= 1.0
+        self.rgb = [
+            (a, b, b),
+            (b, a, b),
+            (b, b, a),
+            (a, a, b),
+            (a, b, a),
+            (b, a, a),
+        ]
 
     def choose(self, i, n):
         """Choose a colour.
@@ -271,7 +282,7 @@ class ColourChooser:
 class NickColourizer:
     """Choose distinguishable colours for nicknames."""
 
-    def __init__(self, maxnicks=30, colour_chooser=None, default_colours=None):
+    def __init__(self, maxnicks=30, colour_chooser=None):
         """Create a colour chooser for nicknames.
 
         If you know how many different nicks there might be, specify that
@@ -279,10 +290,6 @@ class NickColourizer:
 
         If you really want to, you can specify a colour chooser.  Default is
         ColourChooser().
-
-        If you want, you can specify default colours for certain nicknames
-        (`default_colours` is a mapping of nicknames to HTML colours, that is
-        '#rrggbb' strings).
         """
         if colour_chooser is None:
             colour_chooser = ColourChooser()
@@ -290,8 +297,6 @@ class NickColourizer:
         self.nickcount = 0
         self.maxnicks = maxnicks
         self.nick_colour = {}
-        if default_colours:
-            self.nick_colour.update(default_colours)
 
     def __getitem__(self, nick):
         colour = self.nick_colour.get(nick)
@@ -365,22 +370,29 @@ class AbstractStyle(object):
     description = "Single-line description"
     charset = 'US-ASCII'
 
-    def __init__(self, outfile, colours=None, charset=None):
+    def __init__(self, outfile, colours=None):
         """Create a text formatter for writing to outfile.
 
-        `colours` may have the following attributes:
-           part
-           join
-           server
-           nickchange
-           action
+        The ``colours`` dictionary may have the following items:
+
+        - part
+        - join
+        - server
+        - nickchange
+        - action
+
         """
-        if charset is not None:
-            self.charset = charset
         self.outfile = io.TextIOWrapper(outfile, encoding=self.charset,
                                         errors='xmlcharrefreplace',
                                         line_buffering=True)
         self.colours = colours or {}
+        self._anchors = set()
+
+    def __del__(self):
+        """Destructor to make sure we don't close outfile prematurely."""
+        if not self.outfile.closed:
+            self.outfile.flush()
+            self.outfile.detach()  # don't let TextIOWrapper.__del__ close it!
 
     def head(self, title, prev=('', ''), index=('', ''), next=('', ''),
              searchbox=False):
@@ -409,6 +421,17 @@ class AbstractStyle(object):
         `htmlcolour` is a string ('#rrggbb').
         """
 
+    def timestamp_anchor(self, time):
+        anchor = 't%s' % time
+        if anchor in self._anchors:
+            org_anchor = anchor
+            for n in itertools.count(2):
+                anchor = '%s-%d' % (org_anchor, n)
+                if anchor not in self._anchors:
+                    break
+        self._anchors.add(anchor)
+        return anchor
+
 
 class SimpleTextStyle(AbstractStyle):
     """Text style with little use of colour"""
@@ -437,8 +460,10 @@ class SimpleTextStyle(AbstractStyle):
     def foot(self):
         print("""
 <br>Generated by irclog2html.py %(VERSION)s by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
- - find it at <a href="http://mg.pov.lt/irclog2html/">mg.pov.lt</a>!
-</tt></body></html>""" % {'VERSION': VERSION, 'RELEASE': RELEASE}, end=' ', file=self.outfile)
+ - find it at <a href="%(HOMEPAGE)s">%(HOMEPAGE)s</a>!
+ </tt></body></html>""" % {'VERSION': VERSION, 'RELEASE': RELEASE,
+                           'HOMEPAGE': escape(HOMEPAGE)},
+              end=' ', file=self.outfile)
 
     def servermsg(self, time, what, text):
         text = escape(text)
@@ -507,8 +532,7 @@ class TableStyle(SimpleTableStyle):
     def _nicktext(self, time, nick, text, htmlcolour):
         print('<tr><th bgcolor="%s"><font color="#ffffff">'
               '<tt>%s</tt></font></th>'
-              '<td width="100%%" bgcolor="#eeeeee"><tt><font'
-                 ' color="%s">%s</font></tt></td></tr>'
+              '<td width="100%%" bgcolor="#eeeeee"><tt><font color="%s">%s</font></tt></td></tr>'
               % (htmlcolour, nick, htmlcolour, text), file=self.outfile)
 
 
@@ -548,8 +572,7 @@ class XHTMLStyle(AbstractStyle):
   <meta name="version" content="%(VERSION)s - %(RELEASE)s" />
 </head>
 <body>""" % {'VERSION': VERSION, 'RELEASE': RELEASE,
-             'title': escape(title), 'charset': self.charset},
-              file=self.outfile)
+             'title': escape(title), 'charset': self.charset}, file=self.outfile)
         self.heading(title)
         if searchbox:
             self.searchbox()
@@ -578,6 +601,7 @@ class XHTMLStyle(AbstractStyle):
 </form>
 </div>
 """, file=self.outfile)
+
     def navbar(self, prev, index, next):
         prev_title, prev_url = prev
         index_title, index_url = index
@@ -597,10 +621,11 @@ class XHTMLStyle(AbstractStyle):
         print("""
 <div class="generatedby">
 <p>Generated by irclog2html.py %(VERSION)s by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
- - find it at <a href="http://mg.pov.lt/irclog2html/">mg.pov.lt</a>!</p>
+ - find it at <a href="%(HOMEPAGE)s">%(HOMEPAGE)s</a>!</p>
 </div>
 </body>
-</html>""" % {'VERSION': VERSION, 'RELEASE': RELEASE}, file=self.outfile)
+</html>""" % {'VERSION': VERSION, 'RELEASE': RELEASE,
+              'HOMEPAGE': escape(HOMEPAGE)}, file=self.outfile)
 
     def servermsg(self, time, what, text):
         """Output a generic server message.
@@ -612,15 +637,20 @@ class XHTMLStyle(AbstractStyle):
         text = escape(text)
         text = createlinks(text)
         if time:
-            displaytime = shorttime(time)
-            print('<p id="t%s" class="%s">'
-                  '<a href="#t%s" class="time">%s</a> '
-                  '%s</p>'
-                  % (time, self.CLASSMAP[what], time, displaytime, text),
-                  file=self.outfile)
+            print(
+                '<p id="{anchor}" class="{css_class}">'
+                '<a href="#{anchor}" class="time">{time}</a>'
+                ' {text}</p>'.format(
+                    anchor=self.timestamp_anchor(time),
+                    css_class=self.CLASSMAP[what],
+                    time=shorttime(time),
+                    text=text),
+                file=self.outfile)
         else:
-            print('<p class="%s">%s</p>' % (self.CLASSMAP[what], text),
-                  file=self.outfile)
+            print(
+                '<p class="{css_class}">{text}</p>'.format(
+                    css_class=self.CLASSMAP[what], text=text),
+                file=self.outfile)
 
     def nicktext(self, time, nick, text, htmlcolour):
         """Output a comment uttered by someone.
@@ -634,20 +664,28 @@ class XHTMLStyle(AbstractStyle):
         text = createlinks(text)
         text = text.replace('  ', '&nbsp;&nbsp;')
         if time:
-            displaytime = shorttime(time)
-            print('<p id="t%s" class="comment">'
-                  '<a href="#t%s" class="time">%s</a> '
-                  '<span class="nick" style="color: %s">'
-                  '&lt;%s&gt;</span>'
-                  ' <span class="text">%s</span></p>'
-                  % (time, time, displaytime, htmlcolour, nick, text),
-                  file=self.outfile)
+            print(
+                '<p id="{anchor}" class="comment">'
+                '<a href="#{anchor}" class="time">{time}</a> '
+                '<span class="nick" style="color: {color}">'
+                '&lt;{nick}&gt;</span>'
+                ' <span class="text">{text}</span></p>'.format(
+                    anchor=self.timestamp_anchor(time),
+                    time=shorttime(time),
+                    color=htmlcolour,
+                    nick=nick,
+                    text=text),
+                file=self.outfile)
         else:
-            print('<p class="comment">'
-                  '<span class="nick" style="color: %s">'
-                  '&lt;%s&gt;</span>'
-                  ' <span class="text">%s</span></p>'
-                  % (htmlcolour, nick, text), file=self.outfile)
+            print(
+                '<p class="comment">'
+                '<span class="nick" style="color: {color}">'
+                '&lt;{nick}&gt;</span>'
+                ' <span class="text">{text}</span></p>'.format(
+                    color=htmlcolour,
+                    nick=nick,
+                    text=text),
+                file=self.outfile)
 
 
 class XHTMLTableStyle(XHTMLStyle):
@@ -663,18 +701,25 @@ class XHTMLTableStyle(XHTMLStyle):
         text = escape(text)
         text = createlinks(text)
         if time:
-            displaytime = shorttime(time)
-            print('<tr id="t%s">'
-                  '<td class="%s" colspan="2">%s</td>'
-                  '<td><a href="%s#t%s" class="time">%s</a></td>'
-                  '</tr>'
-                  % (time, self.CLASSMAP[what], text, link, time, displaytime),
-                  file=self.outfile)
+            print(
+                '<tr id="{anchor}">'
+                '<td class="{css_class}" colspan="2">{text}</td>'
+                '<td><a href="{link}#{anchor}" class="time">{time}</a></td>'
+                '</tr>'.format(
+                    anchor=self.timestamp_anchor(time),
+                    css_class=self.CLASSMAP[what],
+                    text=text,
+                    link=link,
+                    time=shorttime(time)),
+                file=self.outfile)
         else:
-            print('<tr>'
-                  '<td class="%s" colspan="3">%s</td>'
-                  '</tr>'
-                  % (self.CLASSMAP[what], text), file=self.outfile)
+            print(
+                '<tr>'
+                '<td class="{css_class}" colspan="3">{text}</td>'
+                '</tr>'.format(
+                    css_class=self.CLASSMAP[what],
+                    text=text),
+                file=self.outfile)
 
     def nicktext(self, time, nick, text, htmlcolour, link=''):
         nick = escape(nick)
@@ -682,21 +727,30 @@ class XHTMLTableStyle(XHTMLStyle):
         text = createlinks(text)
         text = text.replace('  ', '&nbsp;&nbsp;')
         if time:
-            displaytime = shorttime(time)
-            print('<tr id="t%s">'
-                  '<th class="nick" style="background: %s">%s</th>'
-                  '<td class="text" style="color: %s">%s</td>'
-                  '<td class="time">'
-                  '<a href="%s#t%s" class="time">%s</a></td>'
-                  '</tr>'
-                  % (time, htmlcolour, nick, htmlcolour, text,
-                     link, time, displaytime), file=self.outfile)
+            print(
+                '<tr id="{anchor}">'
+                '<th class="nick" style="background: {color}">{nick}</th>'
+                '<td class="text" style="color: {color}">{text}</td>'
+                '<td class="time">'
+                '<a href="{link}#{anchor}" class="time">{time}</a></td>'
+                '</tr>'.format(
+                    anchor=self.timestamp_anchor(time),
+                    color=htmlcolour,
+                    nick=nick,
+                    text=text,
+                    link=link,
+                    time=shorttime(time)),
+                file=self.outfile)
         else:
-            print('<tr>'
-                  '<th class="nick" style="background: %s">%s</th>'
-                  '<td class="text" colspan="2" style="color: %s">%s</td>'
-                  '</tr>'
-                  % (htmlcolour, nick, htmlcolour, text), file=self.outfile)
+            print(
+                '<tr>'
+                '<th class="nick" style="background: {color}">{nick}</th>'
+                '<td class="text" colspan="2" style="color: {color}">{text}</td>'
+                '</tr>'.format(
+                    color=htmlcolour,
+                    nick=nick,
+                    text=text),
+                file=self.outfile)
 
 
 class MediaWikiStyle(AbstractStyle):
@@ -743,9 +797,8 @@ class MediaWikiStyle(AbstractStyle):
     def foot(self):
         print('|}\n\nGenerated by irclog2html.py %(VERSION)s '
               'by [mailto:marius@pov.lt Marius Gedminas] - '
-              'find it at [http://mg.pov.lt/irclog2html '
-              'mg.pov.lt]!'
-              % {'VERSION': VERSION}, file=self.outfile)
+              'find it at [%(HOMEPAGE)s %(HOMEPAGE)s]!'
+              % {'VERSION': VERSION, 'HOMEPAGE': HOMEPAGE}, file=self.outfile)
 
 
 #
@@ -791,10 +844,9 @@ def do_config_file(option, opt_str, value, parser):
 
 
 def parse_args(argv=sys.argv):
-    progname = os.path.basename(argv[0])
     parser = optparse.OptionParser("usage: %prog [options] filename [...]",
-                                   prog=progname,
                                    version=VERSION,
+                                   prog='irclog2html',
                                    description="Colourises and converts IRC"
                                                " logs to HTML format for easy"
                                                " web reading.")
@@ -884,7 +936,8 @@ def main(argv=sys.argv):
         if not options.output_file:
             outfilename = pick_output_filename(filename)
         elif os.path.isdir(options.output_file):
-            outfilename = os.path.join(options.output_file,
+            outfilename = os.path.join(
+                options.output_file,
                 os.path.basename(pick_output_filename(filename)))
         else:
             outfilename = options.output_file
