@@ -9,7 +9,7 @@ import tempfile
 import unittest
 
 from irclog2html.irclog2html import (
-    LogParser, ColourChooser, NickColourizer,
+    LogParser, ColourChooser, NickColourizer, AbstractStyle,
     SimpleTextStyle, TextStyle, SimpleTableStyle, TableStyle,
     XHTMLStyle, XHTMLTableStyle, MediaWikiStyle,
     COLOURS, parse_args, main)
@@ -91,6 +91,8 @@ def doctest_LogParser():
         '02-Feb-2004 14:18:55' COMMENT ('mg', 'Hello!')
         >>> test('[15 Jan 08:42] <mg> +++Hello+++')
         '15 Jan 08:42' COMMENT ('mg', '+++Hello+++')
+        >>> test('1075917300 <mg> Hello!')
+        '2004-02-04T17:55:00' COMMENT ('mg', 'Hello!')
 
     Excessive metainformation is stripped from nicknames
 
@@ -286,6 +288,22 @@ class BytesIOWrapper(object):
         self.stream.flush()
 
 
+def doctest_AbstractStyle_timestamp_anchor_duplicate_timestamps():
+    """Test for AbstractStyle.timestamp_anchor
+
+        >>> style = AbstractStyle(BytesIOWrapper(sys.stdout))
+        >>> print(style.timestamp_anchor('14:51:30'))
+        t14:51:30
+        >>> print(style.timestamp_anchor('14:51:31'))
+        t14:51:31
+        >>> print(style.timestamp_anchor('14:51:31'))
+        t14:51:31-2
+        >>> print(style.timestamp_anchor('14:51:31'))
+        t14:51:31-3
+
+    """
+
+
 def doctest_SimpleTextStyle():
     """Test for SimpleTextStyle
 
@@ -316,7 +334,7 @@ def doctest_SimpleTextStyle():
         >>> style.foot()
         <BLANKLINE>
         <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
-         - find it at <a href="http://mg.pov.lt/irclog2html/">mg.pov.lt</a>!
+         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
         </tt></body></html>
 
     """
@@ -353,7 +371,7 @@ def doctest_SimpleTextStyle_colourful():
         >>> style.foot()
         <BLANKLINE>
         <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
-         - find it at <a href="http://mg.pov.lt/irclog2html/">mg.pov.lt</a>!
+         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
         </tt></body></html>
 
     """
@@ -390,7 +408,7 @@ def doctest_TextStyle_colourful():
         >>> style.foot()
         <BLANKLINE>
         <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
-         - find it at <a href="http://mg.pov.lt/irclog2html/">mg.pov.lt</a>!
+         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
         </tt></body></html>
 
     """
@@ -428,7 +446,7 @@ def doctest_SimpleTableStyle():
         </table>
         <BLANKLINE>
         <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
-         - find it at <a href="http://mg.pov.lt/irclog2html/">mg.pov.lt</a>!
+         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
         </tt></body></html>
 
     """
@@ -466,7 +484,7 @@ def doctest_TableStyle():
         </table>
         <BLANKLINE>
         <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
-         - find it at <a href="http://mg.pov.lt/irclog2html/">mg.pov.lt</a>!
+         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
         </tt></body></html>
 
     """
@@ -517,7 +535,7 @@ def doctest_XHTMLStyle():
         <BLANKLINE>
         <div class="generatedby">
         <p>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
-         - find it at <a href="http://mg.pov.lt/irclog2html/">mg.pov.lt</a>!</p>
+         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!</p>
         </div>
         </body>
         </html>
@@ -602,7 +620,7 @@ def doctest_XHTMLTableStyle():
         <BLANKLINE>
         <div class="generatedby">
         <p>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
-         - find it at <a href="http://mg.pov.lt/irclog2html/">mg.pov.lt</a>!</p>
+         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!</p>
         </div>
         </body>
         </html>
@@ -682,7 +700,7 @@ def doctest_MediaWikiStyle():
         >>> style.foot()
         |}
         <BLANKLINE>
-        Generated by irclog2html.py ... by [mailto:marius@pov.lt Marius Gedminas] - find it at [http://mg.pov.lt/irclog2html mg.pov.lt]!
+        Generated by irclog2html.py ... by [mailto:marius@pov.lt Marius Gedminas] - find it at [https://mg.pov.lt/irclog2html/ https://mg.pov.lt/irclog2html/]!
 
     """
 
